
#include <nds.h>
#include <nds/arm9/console.h>
#include "bsc.h"

int main(void)
{	
	int starLimit, i, mag;
	int fov, rx, ry, rz;

	fov = 35;
	rx = ry = rz = 0;
	starLimit = 8192;
	
	// Turn on everything
	powerON(POWER_ALL);
	
	lcdSwap();

	// Setup the Main screen for 3D 
	videoSetMode(MODE_0_3D);
	vramSetBankA(VRAM_A_LCD);
	
	// Setup console screen
	videoSetModeSub(MODE_0_2D | DISPLAY_BG0_ACTIVE);
	vramSetBankC(VRAM_C_SUB_BG);
	SUB_BG0_CR = BG_MAP_BASE(31);
	BG_PALETTE_SUB[255] = RGB15(31,31,31);
	consoleInitDefault((u16*)SCREEN_BASE_BLOCK_SUB(31), (u16*)CHAR_BASE_BLOCK_SUB(0), 16);
	
	// IRQ basic setup
	irqInitHandler(irqDefaultHandler);
	irqSet(IRQ_VBLANK, 0);
	
	// Set our viewport to be the same size as the screen
	glViewPort(0,0,255,191);	
	
	// Specify the Clear Color and Depth 
	glClearColor(0,0,0);
	glClearDepth(0x7FFF);
	
	while (1) 
	{
		// Reset the screen and setup the view
		glReset();
		
		// Increment rotation on 3 axis
		if((!(KEYS & KEY_UP)) && (rx > -256)) rx -= 4;
		if((!(KEYS & KEY_DOWN)) && (rx < 256)) rx += 4;
		if(!(KEYS & KEY_LEFT)) rz -= 4;
		if(!(KEYS & KEY_RIGHT)) rz += 4;
		
		if(!(KEYS & KEY_A)) ry -= 4;
		if(!(KEYS & KEY_B)) ry += 4;
		
		// Set number of stars to display
		if((!(KEYS & KEY_START)) && (starLimit <=36384 )) starLimit += 1024;;
		if((!(KEYS & KEY_SELECT)) && (starLimit > 1024)) starLimit -= 1024;
		
		// Increment Field of View angle
		if((!(KEYS & KEY_R)) && (fov >= 2 )) fov--;
		if((!(KEYS & KEY_L)) && (fov <= 60 )) fov++;

		gluPerspective(fov, 256.0 / 192.0, 1.0, 1024.0);
		
		//ds specific, several attributes can be set here	
		glPolyFmt(POLY_ALPHA(31) | POLY_CULL_NONE);
		
		// Set the current matrix to be the model matrix
		glMatrixMode(GL_MODELVIEW);
		
		//Push our original Matrix onto the stack (save state)
		glPushMatrix();	
		
		glLoadIdentity();
		
		// Set scene
		gluLookAt(	0.0, 0.0, 0.0,
					0.0, 0.0, -1.0,
					0.0, 1.0, 0.0);
		
		for( i = 0; i < starLimit ; i += 4){
			glLoadIdentity();
			// Rotate on 3 axis
			glRotateXi(rx);
			glRotateYi(ry);
			glRotateZi(rz);
			// Translate according to Bright Star Catalogue position
			glTranslate3f32(bsc[i], bsc[i+1], bsc[i+2]);
			// Set a greyscale color according to the star magnitude
			mag = bsc[i+3] >> 12;
			glColor(RGB15(mag, mag, mag));
			// Start Drawing a "point-Triangle", i didn't find the point mode
			glBegin(GL_TRIANGLE);
				glVertex3v16(0, 0, 0);
				glVertex3v16(0, 0, 0);
				glVertex3v16(0, 0, 0);
			glEnd();
			// Done Drawing The point-Triangle			
		}

		// Pop our Matrix from the stack (restore state)
		glPopMatrix(1);

		swiWaitForVBlank();
		// flush to screen	
		glFlush();
	}
}
